
import java.io.File;
import java.io.IOException;

import com.sodius.rhapsody.toolkit.properties.ImportToolKind;

public class XMI4RhapsodyINIUpdate {

    public static final String NAME_PLUGIN_CLASS_EXPORT = "com.sodius.toolkit.rhapsody.launcher.ExportPluginLauncher";
    public static final String NAME_PLUGIN_CLASS_IMPORT = "com.sodius.toolkit.rhapsody.launcher.ImportPluginLauncher";
    public static final String NAME_PLUGIN_NAME_EXPORT = "Export_XMI_from_Rhapsody_PluginMode";
    public static final String NAME_PLUGIN_NAME_IMPORT = "Import_XMI_into_Rhapsody_PluginMode";
    public static final String NAME_PLUGIN_MENU_NAME_EXPORT = "Export XMI from Rhapsody - Plugin Mode";
    public static final String NAME_PLUGIN_MENU_NAME_IMPORT = "Import XMI into Rhapsody - Plugin Mode";

    public static final String NAME_EMBEDDED_NAME = "MDWorbenchOnDemandServer";
    public static final String NAME_EMBEDDED_CLASS = "com.sodius.mdw.metamodel.rhapsody.io.embedded.launcher.ServerPlugin";

    protected String RHP_VERSION;
    protected String RHP_DIR;
    protected String INSTALL_DIR;
    protected String JAVA;
    protected RhapsodyIniFile ini;

    // the path separator
    private final String P = File.pathSeparator;
    private final String S = File.separator;

    public static void main(String[] args) {
        if (args.length != 6) {
            System.err.println("Incorrect number of arguments :" + args.length);
            System.err.println("Usage : XMI4RhapsodyINIUpdate <mode> <java> <WinDIR> <RhapsodyVersion> <XMIToolkit dir> <RHAPS_DIR>");
            System.err.println("<mode> : 'i' for install, 'u' for uninstall");

        } else {
            XMI4RhapsodyINIUpdate inst;
            try {
                inst = new XMI4RhapsodyINIUpdate(args[1], args[2], args[3], args[4], args[5]);
                if (args[0].equals("i")) {
                    inst.install(args[1]);
                } else if (args[0].equals("u")) {
                    inst.uninstall();
                } else if (args[0].equals("iPlugin")) {
                    inst.installPlugin(args[1]);
                } else if (args[0].equals("uPlugin")) {
                    inst.unInstallPlugin();
                } else if (args[0].equals("iEmbedded")) {
                    inst.installEmbedded(args[1]);
                } else if (args[0].equals("uEmbedded")) {
                    inst.unInstallEmbedded();
                } else {
                    System.err.println("Incorrect value for mode parameter");
                    System.err.println("<mode> : 'i' for install, 'u' for uninstall, 'iPlugin' / 'uPlugin' in case of Plugin");
                }
            } catch (Exception e) {
                System.out.println("Error updating Rhapsody.ini.");
                e.printStackTrace();
            }

        }
    }

    public XMI4RhapsodyINIUpdate(String java, String win_dir, String rhap_ver, String install_dir, String rhaps_dir) throws IOException {
        JAVA = java;
        RHP_VERSION = rhap_ver;
        INSTALL_DIR = install_dir;
        RHP_DIR = rhaps_dir;

        ini = new RhapsodyIniFile(win_dir, rhaps_dir, false);

    }

    public void installPlugin(String java) throws IOException {

        String classpath = INSTALL_DIR + S + "lib" + S + "XMI4Rhapsody.jar";

        // add import plugin
        ini.addJavaPlugin(NAME_PLUGIN_NAME_IMPORT, RHP_VERSION, NAME_PLUGIN_MENU_NAME_IMPORT, NAME_PLUGIN_CLASS_IMPORT, classpath,
                RHP_DIR + S + "Share" + S + "JavaAPI" + P, RhapsodyIniFile.MENU);

        // add export plugin
        ini.addJavaPlugin(NAME_PLUGIN_NAME_EXPORT, RHP_VERSION, NAME_PLUGIN_MENU_NAME_EXPORT, NAME_PLUGIN_CLASS_EXPORT, classpath,
                RHP_DIR + S + "Share" + S + "JavaAPI" + P, RhapsodyIniFile.MENU);

        // set the jvm location
        String jvmLocation = ini.getValue("JVM", "JavaLocation");
        if ((jvmLocation == null) || jvmLocation.equals("")) {
            // linux passes us the "bin" dir
            jvmLocation = new File(java).getParent();
            // we need to set it
            ini.setValue("JVM", "JavaLocation", jvmLocation);
            ini.setValue("JVM", "Options", "ClassPath,LibPath");
            ini.setValue("JVM", "LibPath", "-Djava.library.path=" + RHP_DIR + S + "Share" + S + "JavaAPI" + P);
            ini.setValue("JVM", "ClassPath", "-Djava.class.path=" + RHP_DIR + S + "Share" + S + "JavaAPI" + S + "rhapsody.jar");
        }
        ini.saveFile();
    }

    public void installEmbedded(String java) throws IOException {

        String classpath = INSTALL_DIR + S + "server" + S + "com.sodius.mdw.metamodel.rhapsody.io.embedded.launcher.jar";

        // add import plugin
        ini.addJavaPlugin(NAME_EMBEDDED_NAME, RHP_VERSION, "", NAME_EMBEDDED_CLASS, classpath, "", null);

        ini.saveFile();
    }

    public void unInstallPlugin() throws IOException {

        ini.removePlugin(NAME_PLUGIN_NAME_EXPORT, RHP_VERSION);
        ini.removePlugin(NAME_PLUGIN_NAME_IMPORT, RHP_VERSION);

        ini.saveFile();
    }

    public void unInstallEmbedded() throws IOException {

        ini.removePlugin(NAME_EMBEDDED_NAME, RHP_VERSION);

        ini.saveFile();
    }

    public void install(String java) throws IOException {
        String applicationName = "";
        if (System.getProperty("os.name").startsWith("Windows")) {
            applicationName = "XMI4Rhapsody.exe";
        } else {
            applicationName = "toolkit.sh";
        }

        String importCommand = "\"" + INSTALL_DIR + S + applicationName + "\"";
        String importInitialDir = "\"" + INSTALL_DIR + "\"";
        String importDllCompatible = "1";
        String importIsVisible = "1";
        String importIsMacro = "0";
        for (ImportToolKind importTool : ImportToolKind.values()) {
            String importName = getImportName(importTool);
            String importArguments = "-mode IMPORT" + (importTool.format != null ? " -format " + importTool.format : "");
            ini.addHelper(importName, importCommand, importArguments, importInitialDir, importDllCompatible, importIsVisible, importIsMacro, "");
        }

        String exportName = "Export XMI from Rhapsody";
        String exportCommand = "\"" + INSTALL_DIR + S + applicationName + "\"";
        String exportArguments = "-mode EXPORT";
        String exportInitialDir = "\"" + INSTALL_DIR + "\"";
        String exportDllCompatible = "1";
        String exportIsVisible = "1";
        String exportIsMacro = "0";

        ini.addHelper(exportName, exportCommand, exportArguments, exportInitialDir, exportDllCompatible, exportIsVisible, exportIsMacro, "");

        ini.saveFile();
    }

    private String getImportName(ImportToolKind tool) {
        return "Import\\From " + tool.name;
    }

    public void uninstall() throws IOException {

        for (ImportToolKind importTool : ImportToolKind.values()) {
            ini.removeHelper(getImportName(importTool));
        }
        ini.removeHelper("Export XMI from Rhapsody");

        ini.saveFile();
    }

}
