package com.sodius.mdw.log.context;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EStructuralFeature;

import com.sodius.mdw.core.model.Model;


/**
 * @author Mickael Albert
 * Context describing a EStructuralFeature of a element from a Model
 */
public class FeaturedElementContext extends ModelElementContext {

	public static final String KEY_FEATURE="feature";
	protected static final List<String> keys = new ArrayList<String>(Arrays.asList(new String[]{KEY_FEATURE}));
	static{
		keys.addAll(ModelElementContext.keys);
	}

	public FeaturedElementContext(Model model, EClass eClass, EStructuralFeature feature,
			String namespace) {
		super(model, eClass, namespace);
		key2value.put(KEY_FEATURE,feature);
	}
	
	@Override
	public String getValueDescription(String key) {
		if(KEY_FEATURE.equals(key) && hasValue(key))
			return getFeature().getName();
		return super.getValueDescription(key);
	}
	
	public static FeaturedElementContext create(Model model, EClass eClass, EStructuralFeature feature, String namespace)
	{
		return new FeaturedElementContext(model, eClass, feature, namespace);
	}
	
	@Override
	public String toString() {
		return super.toString()+"."+getValueDescription(KEY_FEATURE);
	}

	@Override
	public String getSimpleName() {
		return super.getSimpleName()+"."+getValueDescription(KEY_FEATURE);
	}
	
	protected EStructuralFeature getFeature()
	{
		return (EStructuralFeature)key2value.get(KEY_FEATURE);
	}
}
