package com.sodius.mdw.log.context;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import com.sodius.mdw.core.model.Model;
import com.sodius.mdw.core.model.io.ConnectorDescriptor;

/**
 * @author Mickael Albert
 * Context describing a Model
 */
public class ModelContext extends GenericEventContext {

	public static final String KEY_MODEL="MODEL";
	public static final String KEY_CONNECTOR="CONNECTOR";
	
	protected static final List<String> keys = Arrays.asList(new String[]{KEY_MODEL});
	
	public ModelContext(Model model) {
		this.key2value.put(KEY_MODEL, model);
		if(model!=null)
		{
			if(model.getWriter()!=null)
				key2value.put(KEY_CONNECTOR,model.getWriter());
			else
				key2value.put(KEY_CONNECTOR,model.getReader());
		}
	}

	@Override
	public Collection<String> getKeys() {
		return keys;
	}

	@Override
	public String getSimpleName() {
		if(hasValue(KEY_MODEL))
		{
			return getModel().getMetamodel().getName();
		}
			
		return super.getSimpleName();
	}
	
	@Override
	public String getValueDescription(String key) {
		if(KEY_CONNECTOR.equals(key))
		{
			if(getConnector()!=null) 
				return (getConnector().isReader()?" Reader:":" Writer:")+getConnector().getName();
			else if (getModel() != null)
				return "new "+getModel().getMetamodel().getName();
			else
				return "unknown connector";
		}
		if(KEY_MODEL.equals(key) && hasValue(key))
		{
			Model model=getModel();
			String name=model.getMetamodel().getName();
			if(model.getReadURI()!=null)
				name+="{" + model.getReadURI() + "}";
		}
			
		return super.getValueDescription(key);
	}

	@Override
	public String toString() {
		return getValueDescription(KEY_MODEL)+(hasValue(KEY_CONNECTOR)?"@"+getValueDescription(KEY_CONNECTOR):"");
	}
	
	public static ModelContext create(Model model)
	{
		return new ModelContext(model);
	}
	
	protected Model getModel()
	{
		return (Model)key2value.get(KEY_MODEL);
	}
	
	protected ConnectorDescriptor getConnector()
	{
		return (ConnectorDescriptor)key2value.get(KEY_CONNECTOR);
	}
	
}
