package com.sodius.mdw.log.context;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.eclipse.emf.ecore.EClass;

import com.sodius.mdw.core.model.Model;

/**
 * @author Mickael Albert
 * Context describing an EObject of a Model, through its EClass and Namespace
 */
public class ModelElementContext extends ModelContext {

	private String[] splitNamespace=null;
	public static final String NAMESPACE_SEPARATOR="::";
	public static final String KEY_ECLASS="ECLASS";
	public static final String KEY_NAMESPACE="NAMESPACE";
	protected static final List<String> keys = new ArrayList<String>(Arrays.asList(new String[]{KEY_ECLASS, KEY_NAMESPACE}));
	static{
		keys.addAll(ModelContext.keys);
	}
	
	public ModelElementContext(Model model, EClass eClass, String namespace) {
		super(model);
		key2value.put(KEY_ECLASS,eClass);
		key2value.put(KEY_NAMESPACE,namespace);
	}
	
	public Collection<String> getKeys() {
		return keys;
	}
	
	@Override
	public Object getValue(String key) {
		if(KEY_NAMESPACE.equals(key))
			return getSplitNamespace();
		else
			return super.getValue(key);
	}
	
	/**
	 * @return an String array of namespaces, null if not set
	 */
	private String[] getSplitNamespace()
	{
		if(splitNamespace==null && getNamespace()!=null)
			splitNamespace=getNamespace().split(NAMESPACE_SEPARATOR);
		return splitNamespace;
			
	}

	@Override
	public String getValueDescription(String key) {
		if(KEY_ECLASS.equals(key) && getEClass()!=null)
			return getEClass().getEPackage().getName()+"."+getEClass().getName();
		if(KEY_NAMESPACE.equals(key) && getNamespace()!=null)
			return getNamespace();
		return super.getValueDescription(key);
	}
	
	public static ModelElementContext create(Model model, EClass eClass, String namespace)
	{
		return new ModelElementContext(model, eClass, namespace);
	}
	
	@Override
	public String toString() {
		return "("+super.toString()+")"+getNamespace();
	}
	
	@Override
	public String getSimpleName() {
		String[] names=getSplitNamespace();
		if(names==null)
			return super.getSimpleName();
		return getValueDescription(KEY_ECLASS)+ (names.length>0?":"+names[names.length-1]:"");
	}
	
	protected EClass getEClass()
	{
		return (EClass)key2value.get(KEY_ECLASS);
	}
	
	protected String getNamespace()
	{
		return (String)key2value.get(KEY_NAMESPACE);
	}
	
	
}
