package com.sodius.mdw.log.reporter.tree;

import com.sodius.mdw.log.EventStatus;
import com.sodius.mdw.log.context.ModelElementContext;

/**
 * @author Mickael Albert
 *
 * Generates subtrees following Model Element namespaces.
 * Structure:
 * 1)CONNECTOR
 * 1.1)RootOwner
 * 1.1.1)SubElement
 */
public class ModelSubTreeGenerator extends SubTreeGenerator
{
	private final static String SORT_NAME="Elements qualified name";
	public final static String NO_NAMESPACE="Anonymous";	
	public ModelSubTreeGenerator() {
		super();
	}

	public ModelSubTreeGenerator(boolean displayUnnamedGroup) {
		super(displayUnnamedGroup);
	}

	@Override
	public boolean canGenerateSubTrees(Tree tree) {
		int nextGroupLevel=tree.getGroupLevel()+1;
		int nsLevel=getNamespaceLevel(nextGroupLevel);
		for (EventStatus status : tree.getContainedStatus()) {
			if(status.getContext() instanceof ModelElementContext)
			{
				if(nextGroupLevel==0) //the level is for connector name
					return true;
				if(status.getContext().hasValue(ModelElementContext.KEY_NAMESPACE) && //or the tree can be split to namespaces
						((String[])status.getContext().getValue(ModelElementContext.KEY_NAMESPACE)).length>nsLevel)
					return true;
			}
		}
		return false;
	}

	@Override
	protected String getContent(String groupID, int groupLevel, EventStatus status) {
		return groupID;
	}
	
	@Override
	protected String getQualifiedName(String groupID, int groupLevel,
			EventStatus status) {
		if(status.getContext() instanceof ModelElementContext)
		{
			if(groupLevel==0 || !status.getContext().hasValue(ModelElementContext.KEY_NAMESPACE))
				return groupID;
			String[] namespaces = (String[])status.getContext().getValue(ModelElementContext.KEY_NAMESPACE);
			int nsLevel=getNamespaceLevel(groupLevel);
			if(namespaces.length>nsLevel)
			{
				StringBuilder sb = new StringBuilder();
				for (int i = 0; i <= nsLevel; i++) {
					if(sb.length()>0)
						sb.append(ModelElementContext.NAMESPACE_SEPARATOR);
					sb.append(namespaces[i]);
				}
				return sb.toString();
			}
		}
		return "";
	}

	@Override
	protected String getComment(String groupID, int groupLevel, EventStatus status) {
		return getQualifiedName(groupID, groupLevel, status);
	}

	@Override
	protected String getGroupID(EventStatus status, int groupLevel) {
		if(status.getContext() instanceof ModelElementContext)
		{
			if(groupLevel==0)
				return status.getContext().getValueDescription(ModelElementContext.KEY_CONNECTOR);
			else if(!status.getContext().hasValue(ModelElementContext.KEY_NAMESPACE))
			{
				return NO_NAMESPACE;
			}
			else
			{
				String[] namespaces = (String[])status.getContext().getValue(ModelElementContext.KEY_NAMESPACE);
				int nsLevel=getNamespaceLevel(groupLevel);
				if(namespaces.length>nsLevel)
					return namespaces[nsLevel];
			}
		}
		return null;
	}
	
	private int getNamespaceLevel(int groupLevel)
	{
		return groupLevel-1;
	}
	
	@Override
	public String getSortName() {
		return SORT_NAME;
	}
}