package com.sodius.rhapsody.toolkit.engine;

import java.io.File;
import java.io.IOException;

import com.sodius.mdw.core.util.log.FileLogger;
import com.sodius.mdw.core.util.log.Logger;
import com.sodius.rhapsody.toolkit.properties.RhapsodyXMIToolkitParameters;
import com.sodius.rhapsody.toolkit.util.NullLogger;
import com.sodius.rhapsody.toolkit.util.PrintStreamLogger;

public class HeadlessEngineContext implements EngineContext {
	
	private RhapsodyXMIToolkitParameters parameters;
	private Logger outLogger;
	private Logger errLogger;
	private XMIEngine engine=null;
	
	public HeadlessEngineContext(RhapsodyXMIToolkitParameters parameters) {
		this.parameters = parameters;
		outLogger = this.buildStandardLogger("system_out.txt", "Standard Output Logger");
        errLogger = this.buildStandardLogger("system_err.txt", "Standard Error Logger");
		setSystemStreams();
	}
	
	public RhapsodyXMIToolkitParameters getParameters() {
		return parameters;
	}
	
	public Logger getErrLogger() {
		return outLogger;
	}
	
	public Logger getOutLogger() {
		return errLogger;
	}
	
	public Logger getDirectOutLogger() {
		return outLogger;
	}
	
    private Logger buildStandardLogger(String filename, String label) {
        Logger logLogger = new NullLogger();
        try {
            logLogger = new FileLogger(new File(filename));
        } catch (IOException e) {
            e.printStackTrace();
        }

        return logLogger;
    }

    private void setSystemStreams() {
        try {
            // out
            System.setOut(new PrintStreamLogger(outLogger, false));

            // err
            System.setErr(new PrintStreamLogger(errLogger, true));
        } 
        catch (Throwable e) {
            e.printStackTrace();
        }
    }

	public XMIEngine getXMIEngine() {
		return engine;
	}

	public void setXMIEngine(XMIEngine engine) {
		this.engine=engine;
	}
}
