package com.sodius.rhapsody.toolkit.engine;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.core.model.Metamodel;
import com.sodius.mdw.core.util.io.IOUtils;
import com.sodius.mdw.corext.model.io.xmi.XMIHeader;
import com.sodius.mdw.metamodel.uml21.resource.UML22UMLVersionRegistry;
import com.sodius.mdw.metamodel.uml21.resource.UMLVersionRegistry;
import com.sodius.mdw.metamodel.uml21.resource.XMI2UMLVersionRegistry;
import com.sodius.rhapsody.toolkit.properties.RhapsodyXMIToolkitParameters;

public class MetamodelChecker {

    private static List<String> handledUML2Namespaces; // @See Mantis 0003291
    static {
        handledUML2Namespaces = new ArrayList<String>();
        handledUML2Namespaces.addAll(XMI2UMLVersionRegistry.SET_UML_METAMODEL_NS); // .XMI files
        handledUML2Namespaces.addAll(UML22UMLVersionRegistry.SET_UML_METAMODEL_NS); // .UML2 files
        handledUML2Namespaces.addAll(UMLVersionRegistry.SET_UML_METAMODEL_NS);// .UML files
    }

    private final EngineContext context;

    public MetamodelChecker(EngineContext context) {
        this.context = context;
    }

    private RhapsodyXMIToolkitParameters getProperties() {
        return context.getParameters();
    }

    public Metamodel setSelectedMetamodelFromFile() throws CoreException {

        String declaredName = "";
        String declaredVersion = "";
        String mmName[] = getMetamodelNamedInXMI();
        if (mmName != null) {
            declaredName = mmName[0];
            declaredVersion = mmName[1];
        }
        /*
         * String declaredName = XMIHeader.read(URI.createFileURI(getProperties().getXMIFilename())).getMetamodelName();
         * String declaredVersion = XMIHeader.read(URI.createFileURI(getProperties().getXMIFilename())).getMetamodelVersion();
         */
        Metamodel mm = getProperties().getMetamodelNamed(declaredName, declaredVersion);

        if (mm == null) {
            String errorMessage = "Warning! The XMI file does not declare any accepted metamodel.";
            if (mmName != null) {
                errorMessage = errorMessage + " The Metamodel declared is " + declaredName + "(" + declaredVersion + ")";
            }
            context.getOutLogger().error(errorMessage);
            mm = context.getParameters().getSelectedMetamodel();
        } else {
            if (getProperties().getSelectedMetamodel() != mm) {
                context.getOutLogger()
                        .warning("Warning! The XMI file declares a different metamodel (" + mm.getUniqueID() + ") than the one selected ("
                                + this.getProperties().getSelectedMetamodel().getUniqueID() + "). The importer is changed to " + mm.getUniqueID());
            }
            getProperties().setSelectedMetamodel(mm);
        }

        return mm;
    }

    private String[] getMetamodelNamedInXMI() throws CoreException {
        return getMetamodelNamedInXMI(getProperties().getXMIFilename());
    }

    public String[] getMetamodelNamedInXMI(String xmiFileName) throws CoreException {
        InputStream is = null;
        XMIHeader header = null;
        try {
            is = new BufferedInputStream(new FileInputStream(xmiFileName));
            header = new RhpXMIHeaderHandler().readHeader(is);
            if (header == null) {
                throw new CoreException(xmiFileName + " is not a XMI file");
            }
        } catch (FileNotFoundException e) {
            throw new CoreException(xmiFileName + " does not exist");
        } catch (IOException e) {
            throw new CoreException(xmiFileName + " cannot be read");
        } finally {
            if (is != null) {
                IOUtils.closeQuietly(is);
            }
        }
        if (header == null) {
            return null;
        }

        if ((header.getMetamodelName() != null) && (header.getMetamodelVersion() != null) && (header.getMetamodelName().length() > 0)
                && (header.getMetamodelVersion().length() > 0)) {

            String returnValue[] = { header.getMetamodelName(), header.getMetamodelVersion() };
            return returnValue;
        }
        if ((header.getNamespacePrefix() != null) && (header.getNamespaceURI() != null) && (header.getNamespacePrefix().length() > 0)
                && (header.getNamespaceURI().length() > 0)) {
            if (header.getNamespacePrefix().equals("uml") && handledUML2Namespaces.contains(header.getNamespaceURI())) {
                if ("Enterprise Architect".equals(header.getDocExporter())) {
                    return new String[] { "UML", "2.1", "EA" }; // @See RTC 132921
                }
                if ("MagicDraw UML".equals(header.getDocExporter())) {
                    return new String[] { "UML", "2.1", "MD" };
                }
                return new String[] { "UML", "2.1" };
            } else {
                String returnValue[] = { header.getNamespacePrefix(), header.getNamespaceURI() };
                return returnValue;
            }
        }
        return null;
    }

}
