package com.sodius.rhapsody.toolkit.engine.uml21;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.ecore.resource.Resource;

import com.sodius.mdw.core.CoreException;
import com.sodius.mdw.core.MDWorkbench;
import com.sodius.mdw.core.model.MDWList;
import com.sodius.mdw.core.model.Model;
import com.sodius.mdw.core.model.io.ConnectorDescriptor;
import com.sodius.mdw.core.util.io.IOUtils;
import com.sodius.mdw.metamodel.rhapsody.Component;
import com.sodius.mdw.metamodel.rhapsody.ModelElement;
import com.sodius.mdw.metamodel.rhapsody.Profile;
import com.sodius.mdw.metamodel.rhapsody.Project;
import com.sodius.mdw.metamodel.rhapsody.SequenceDiagram;
import com.sodius.mdw.metamodel.rhapsody.Tag;
import com.sodius.mdw.metamodel.rhapsody.Unit;
import com.sodius.mdw.metamodel.uml21.Namespace;
import com.sodius.mdw.metamodel.uml21.Stereotype;
import com.sodius.rhapsody.toolkit.engine.EngineContext;
import com.telelogic.rhapsody.core.IRPApplication;
import com.telelogic.rhapsody.core.IRPHyperLink;
import com.telelogic.rhapsody.core.IRPModelElement;
import com.telelogic.rhapsody.core.IRPPackage;
import com.telelogic.rhapsody.core.IRPProject;
import com.telelogic.rhapsody.core.IRPTag;
import com.telelogic.rhapsody.core.IRPUnit;
import com.telelogic.rhapsody.core.RhapsodyRuntimeException;

public class RhapsodyWriter {

    private final static String HYPERLINK_NAME = "registeredURLUMLModel";
    private final static String TAG_NAME = "XMIId";

    // @see RTC 124773
    public static void launchDiffMerge(EngineContext context, IRPApplication rhpApp, MDWorkbench mdw, Model umlModel, Model rhaModel, File userTmpDir,
            Map<String, Object> writingOptions) throws CoreException, IOException {
        IRPProject userProject = rhpApp.activeProject();
        IRPProject tmpProject = writeInSeparateProject(context, rhpApp, mdw, userProject, rhaModel, userTmpDir, isFromRhapsody(umlModel),
                writingOptions);

        tmpProject.save();
        String tmpProjectPath = tmpProject.getCurrentDirectory() + File.separator + tmpProject.getFilename();
        String userProjectPath = userProject.getCurrentDirectory() + File.separator + userProject.getFilename();
        userProject.close();

        try {
            Runtime r = Runtime.getRuntime();
            Process p = r.exec(new File(rhpApp.getOMROOT()).getParent() + File.separator + "DiffMerge.exe \"" + userProjectPath + "\" \""
                    + tmpProjectPath + "\" -xcompare");
            p.waitFor();// si l'application doit attendre a ce que ce process fini
        } catch (Exception e) {
            System.out.println("erreur " + e.toString());
        }
        rhpApp.insertProject(userProjectPath);
        tmpProject.close();
    }

    public static void addToModel(EngineContext context, IRPApplication rhpApp, MDWorkbench mdw, Model umlModel, Model rhaModel, String namespaces,
            File userTmpDir, Map<String, Object> writingOptions) throws IOException, CoreException {
        List<String> namespaceList = Arrays.asList(namespaces.split("\\,"));
        if (namespaces.length() == 0) {
            namespaceList = new ArrayList<String>();
        }
        IRPProject userProject = rhpApp.activeProject();
        boolean isPackageLevel = !isFromRhapsody(umlModel);
        IRPProject tmpProject = writeInSeparateProject(context, rhpApp, mdw, userProject, rhaModel, userTmpDir, !isPackageLevel, writingOptions);

        MDWList<Tag> urlTags = rhaModel.<Tag> getInstances("Tag").select("name", "lastKnownURL");
        Map<Unit, IRPUnit> mapMDWRHP = new HashMap<Unit, IRPUnit>();
        List<String> unitPathsToSave = new ArrayList<String>();

        // Set<String> unitPathToAdd = new HashSet<String>();

        // exploring each selected element and save the path for accessing them

        // mapMDWRHP.put((Unit)rhaModel.getInstances("Project").first(), tmpProject);

        boolean isHiddenUI = 1 == rhpApp.getIsHiddenUI();
        try {
            rhpApp.allowBrowserRefresh(0);
            if (!isHiddenUI) {
                rhpApp.setHiddenUI(true);
            }
            if (namespaceList.isEmpty()) // all are selected by default
            {
                for (Namespace element : umlModel.<Namespace> getInstances("Namespace", true).select("owner", null)) {
                    manageElementToAdd(element, mapMDWRHP, unitPathsToSave, urlTags, tmpProject, isPackageLevel);
                }
            } else {
                for (Namespace element : umlModel.<Namespace> getInstances("Namespace", true)) {
                    if (namespaceList.contains(element.getQualifiedName())) {
                        manageElementToAdd(element, mapMDWRHP, unitPathsToSave, urlTags, tmpProject, isPackageLevel);
                    }
                }
            }
            try {
                tmpProject.save(1);
            } catch (RhapsodyRuntimeException e) {
                if (!"The operation completed successfully.".equals(e.getMessage())) {
                    throw e;
                }
            }
            tmpProject.close();
            for (String path : unitPathsToSave) {
                try {
                    rhpApp.addToModel(path, 1);
                } catch (RhapsodyRuntimeException e) {
                    if (!"The operation completed successfully.".equals(e.getMessage())) {
                        throw e;
                    }
                }
            }
        } finally {
            rhpApp.allowBrowserRefresh(1);
            if (!isHiddenUI) {
                rhpApp.setHiddenUI(false);
            }
        }
    }

    private static void manageElementToAdd(Namespace element, Map<Unit, IRPUnit> mapMDWRHP, List<String> unitPathsToSave, MDWList urlTags,
            IRPProject irpProject, boolean isPackageLevel) {
        // get unit in Rhapsody
        Resource resource = element.eResource();
        if (resource != null) {
            String url = resource.getURI().toFileString() + "#" + resource.getURIFragment(element);

            Tag rTag = (Tag) urlTags.detect("value", url); // TODO : test if it works for uppercases
            if (rTag != null) {
                ModelElement rElement = rTag.getOwner();
                if (rElement instanceof Unit) // TODO : error in other case
                {
                    if ((rElement instanceof Project) && !isPackageLevel) {
                        for (com.sodius.mdw.metamodel.rhapsody.Package sub : (List<com.sodius.mdw.metamodel.rhapsody.Package>) ((Project) rElement)
                                .getPackages()) {
                            exploreAndSave(sub, mapMDWRHP, unitPathsToSave, irpProject, isPackageLevel);
                        }
                        for (Component sub : (List<Component>) ((Project) rElement).getComponents()) {
                            exploreAndSave(sub, mapMDWRHP, unitPathsToSave, irpProject, isPackageLevel);
                        }
                        for (Profile sub : (List<Profile>) ((Project) rElement).getProfiles()) {
                            exploreAndSave(sub, mapMDWRHP, unitPathsToSave, irpProject, isPackageLevel);
                        }
                        for (SequenceDiagram sub : (List<SequenceDiagram>) ((Project) rElement).getSequenceDiagrams()) {
                            exploreAndSave(sub, mapMDWRHP, unitPathsToSave, irpProject, isPackageLevel);
                        }
                    } else {
                        exploreAndSave((Unit) rElement, mapMDWRHP, unitPathsToSave, irpProject, isPackageLevel);
                    }
                }
            }
        }
    }

    private static IRPProject writeInSeparateProject(EngineContext context, IRPApplication rhpApp, MDWorkbench mdw, IRPProject userProject,
            Model rhaModel, File userTmpDir, boolean projectLevel, Map<String, Object> writingOptions) throws IOException, CoreException {
        setXMI2GUIDMap(rhaModel, getXMI2GUIDMap(userProject));
        File tmpDir = File.createTempFile("XMIToolkit", "_RPY");
        tmpDir.delete();
        rhpApp.createAndInsertProject(tmpDir.toString(), "TempProject");
        IRPProject tmpProject = rhpApp.activeProject();

        // prepare connector
        ConnectorDescriptor rapDesc = mdw.getMetamodelManager().getMetamodel("rhapsody").getModelWriterDescriptor("Rhapsody Application");
        Object lastImportGUID = rapDesc.getProperty("importGUID");
        Object lastImportLevel = rapDesc.getProperty("importLevel");
        rapDesc.setProperty("importGUID", "true");
        if (projectLevel) {
            rapDesc.setProperty("importLevel", "Project");
        } else {
            rapDesc.setProperty("importLevel", "Package");
        }
        rhaModel.write("Rhapsody Application", "", writingOptions);
        // rhpWriter.setProperty(IRhapsodyConstants.PROPERTY_APPLICATION, rhpApp);
        // TODO : manage plugin

        rapDesc.setProperty("importGUID", lastImportGUID);
        rapDesc.setProperty("importLevel", lastImportLevel);

        return tmpProject;
    }

    private static void exploreAndSave(Unit unit, Map<Unit, IRPUnit> mapMDWRHP, List<String> unitPathsToSave, IRPProject irpProject,
            boolean isPackageLevel) {
        IRPUnit irpUnit = getRhapsodyEquivalent(unit, mapMDWRHP, irpProject, isPackageLevel);
        if (irpUnit != null) {
            if (shouldSaveIt(irpUnit)) {
                irpUnit.setSeparateSaveUnit(1);
                irpUnit.save(1);
                unitPathsToSave.add(irpUnit.getCurrentDirectory() + File.separator + irpUnit.getFilename());
            }
        }
    }

    private static boolean shouldSaveIt(IRPUnit irpUnit) {
        if (irpUnit.isReferenceUnit() == 1) {
            return false;
        }
        if (irpUnit.getIsExternal() == 1) {
            return false;
        }
        if ("Settings".equals(irpUnit.getUserDefinedMetaClass())) {
            return false;
        }
        return true;
    }

    private static IRPUnit getRhapsodyEquivalent(Unit unit, Map<Unit, IRPUnit> mapMDWRHP, IRPProject irpProject, boolean isPackageLevel) {
        if (!mapMDWRHP.containsKey(unit)) {
            IRPUnit irpUnit = null;
            if (unit instanceof Project) {
                if (isPackageLevel) {
                    irpUnit = getSon(irpProject, unit);
                } else {
                    irpUnit = irpProject;
                }

            } else if ((unit instanceof Profile) && isPackageLevel) {
                irpUnit = getSon(irpProject, unit);
            } else if (unit.getOwner() instanceof Unit) {
                IRPUnit owner = getRhapsodyEquivalent((Unit) unit.getOwner(), mapMDWRHP, irpProject, isPackageLevel);
                if (owner != null) {
                    irpUnit = getSon(owner, unit);
                }
            }
            if (irpUnit != null) {
                mapMDWRHP.put(unit, irpUnit);
            }
        }
        return mapMDWRHP.get(unit);
    }

    private static IRPUnit getSon(IRPUnit owner, Unit mdwSon) {
        IRPModelElement irpSon = owner.findNestedElement(mdwSon.getName(), mdwSon.eMetaTypeName());
        if (irpSon == null) {
            irpSon = owner.findNestedElement(mdwSon.getName(), null);
        }
        if (irpSon == null) {
            // deep search
            Tag tagID = (Tag) mdwSon.getLocalTags().detect("name", TAG_NAME);
            if ((tagID != null) && (tagID.getValue() != null)) {
                List<IRPModelElement> sons = owner.getNestedElements().toList();
                for (IRPModelElement son : sons) {
                    if (son instanceof IRPUnit) {
                        IRPTag tag = son.getTag(TAG_NAME);
                        if ((tag != null) && son.equals(tag.getOwner())) {
                            if (tagID.getValue().equals(tag.getValue())) {
                                irpSon = son;
                                break;
                            }
                        }
                    }
                }
            }
        }
        if (irpSon instanceof IRPUnit) {
            return (IRPUnit) irpSon;
        }
        return null;
    }

    private static void addExplorableElements(Unit unitToSave, Set<Unit> explorableUnits) {
        explorableUnits.add(unitToSave);
        if (unitToSave.getOwner() instanceof Unit) {
            addExplorableElements((Unit) unitToSave.getOwner(), explorableUnits);
        }
    }

    private static boolean deleteRecursive(File folder) {
        File[] subFiles = folder.listFiles();
        for (File subFile2 : subFiles) {
            File subFile = subFile2;
            if (subFile.isDirectory()) {
                deleteRecursive(subFile);
            } else {
                try {
                    if (!subFile.delete()) {
                        ;// Silent sendErrorMessage("Error when trying to delete File " + subFile.getAbsolutePath());
                    }
                } catch (Exception e) {
                    // Silent sendErrorMessage("Error when trying to delete File " + subFile.getAbsolutePath());
                }
            }
        }
        try {
            if (folder.delete()) {
                ;// Silent sendErrorMessage("Error when trying to delete Folder " + folder.getAbsolutePath());
            }
        } catch (Exception e) {
            // Silent sendErrorMessage("Error when trying to delete Folder " + folder.getAbsolutePath());
        }
        return true;
    }

    private static void recursiveFolderCopy(File folderIn, File folderOut) throws IOException {
        if (folderIn.isDirectory()) {
            if (!folderOut.exists()) {
                folderOut.mkdirs();
            }
            for (int i = 0; i < folderIn.listFiles().length; i++) {
                File subFileIn = folderIn.listFiles()[i];
                File subFileOut = new File(folderOut, subFileIn.getName());
                if (subFileIn.isDirectory()) {
                    recursiveFolderCopy(subFileIn, subFileOut);
                } else {
                    IOUtils.copy(subFileIn, subFileOut);
                }
            }
        }
    }

    private static Map<String, String> getXMI2GUIDMap(IRPProject project) {
        Map<String, String> xmi2GuidMap = new HashMap<String, String>();
        try {
            IRPPackage predefinedTypes = (IRPPackage) project.findElementByGUID("GUID 0d7c31da-7afa-4eca-b608-47dcf53db5ec"); // PredefinedTypes GUID
            for (IRPModelElement reference : (List<IRPModelElement>) predefinedTypes.getReferences().toList()) {
                try {
                    if ((reference instanceof IRPHyperLink) && HYPERLINK_NAME.equals(reference.getName())) {
                        String xmi = ((IRPTag) reference.getOwner()).getValue();
                        xmi2GuidMap.put(xmi, reference.getOwner().getOwner().getGUID());
                    }
                } catch (RhapsodyRuntimeException re) {
                    // try next element
                }
            }
        } catch (Exception e) {
            // Still return a map
        }
        return xmi2GuidMap;
    }

    private static void setXMI2GUIDMap(Model rhaModel, Map<String, String> xmi2GuidMap) {
        List<Tag> xmiTags = rhaModel.<Tag> getInstances("Tag").select("name", TAG_NAME);
        for (Tag tag : xmiTags) {
            if ((tag.getOwner() != null) && xmi2GuidMap.containsKey(tag.getValue())) {
                String guid = xmi2GuidMap.get(tag.getValue());
                tag.getOwner().setGUID(guid);
            }
        }
    }

    private static boolean isFromRhapsody(Model umlModel) {
        for (com.sodius.mdw.metamodel.uml21.Model model : umlModel.<com.sodius.mdw.metamodel.uml21.Model> getInstances("Model")) {
            for (Stereotype st : (List<Stereotype>) model.getAppliedStereotypes()) {
                if ("RhpProject".equals(st.getName())) {
                    return true;
                }
            }
        }
        return false;
    }

}
