package com.sodius.rhapsody.toolkit.properties;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;


/**
 * @author xavier
 */
public class RhapsodyPropertiesReader  {

    private String rhapsodyPropertiesFile = "";
    protected Map propertyCache = new HashMap();
    protected Properties rhapsodyProperties;
    private RhapsodyProperty[] allProperties;
    
    static public RhapsodyPropertiesReader EMPTY_PROPERTIES_READER = new RhapsodyPropertiesReader();
    
    protected RhapsodyPropertiesReader() {
        this.rhapsodyProperties = new Properties();
    }

    public RhapsodyPropertiesReader(String rhapsodyPropertiesFile) throws IOException {
        this.rhapsodyPropertiesFile = rhapsodyPropertiesFile;
        this.reloadRhapsodyProperties();
    }

    public void setPropertiesFile(String rhapsodyPropertiesFile) throws IOException {
        this.rhapsodyPropertiesFile = rhapsodyPropertiesFile;
        this.reloadRhapsodyProperties();
    }
    
    public String getPropertiesFile() {
        return rhapsodyPropertiesFile;
    }

    protected void addPropertyForMeta(String metaName, String key, String shortKey) {
        // Note - no checking is done for duplicates
        Collection list = (Collection) propertyCache.get(metaName);
        if (list == null) {
            list = new ArrayList();
            propertyCache.put(metaName, list);
        }
        RhapsodyProperty prop = new RhapsodyProperty();
        prop.setToolkitName(key);
        prop.setRhapsodyName(shortKey);
        prop.setDefaultValue(this.getRhapsodyPropertyDefaultValue(key));
        list.add(prop);
    }
    
    public void reloadRhapsodyProperties() throws IOException {
        synchronized (propertyCache) {
            propertyCache = new HashMap();
            rhapsodyProperties = new Properties();
            //try {
                InputStream is = new FileInputStream(rhapsodyPropertiesFile);
                rhapsodyProperties.load(is);
                is.close();
            //} catch ( FileNotFoundException e ) {         

            //}             
            // cache the entries by metatype as well
            for (Enumeration e = rhapsodyProperties.keys(); e.hasMoreElements(); ) {
                String key = (String) e.nextElement();
                int index = 0;
                if ((index = key.indexOf(".")) != -1 && index != 0) {
                    String metaName = key.substring(0, index);
                    String shortKey = key.substring(index + 1);
                    this.addPropertyForMeta(metaName, key, shortKey);
                }
            }
            // change all lists to arrays
            Iterator entries = propertyCache.entrySet().iterator();
            while (entries.hasNext()) {
                Map.Entry entry = (Map.Entry) entries.next();
                ArrayList list = (ArrayList)entry.getValue();
                propertyCache.put(entry.getKey(), list.toArray(new RhapsodyProperty[list.size()]));
            }
        }
    }

    protected Properties getRhapsodyProperties() {
        if (rhapsodyProperties == null) {
            try {
                reloadRhapsodyProperties();
            }
            catch (IOException e) {
                rhapsodyProperties = new Properties();
            }
        }
        return rhapsodyProperties;
    }


    public RhapsodyProperty[] getRhapsodyPropertiesKeys(String metaclass) {

        Object list = propertyCache.get(metaclass);
        if (list == null) {
            list = new RhapsodyProperty[0];
            propertyCache.put(metaclass, list);
        } 
        return (RhapsodyProperty []) list;
    }


    public String getRhapsodyPropertyDefaultValue(String key) {
        return getRhapsodyProperties().getProperty(key);
    }


    public String getRhapsodyPropertyKey(String key) {
        return key.substring(key.indexOf('.')+1);
    }
    
    public RhapsodyProperty[] getAllProperties() {
        if (allProperties == null) {
            Set properties = new HashSet();
            Collection propArrays = propertyCache.values();
            Iterator propArrayIter = propArrays.iterator();
            while (propArrayIter.hasNext()) {
                Object propArray = propArrayIter.next();
                RhapsodyProperty[] elements = (RhapsodyProperty[]) propArray;
                for (int i = 0; i < elements.length; i++) {
                    properties.add(elements[i]);
                }
            }
            allProperties = (RhapsodyProperty[]) properties.toArray(new RhapsodyProperty[properties.size()]);
        }
        return allProperties;
    }

}
