package com.sodius.rhapsody.toolkit.ui;

import org.eclipse.swt.widgets.Composite;

import com.sodius.mdw.core.CoreException;
import com.sodius.rhapsody.toolkit.properties.RhapsodyXMIToolkitParameters;
import com.sodius.rhapsody.toolkit.properties.UMLVersionKind;

class ExportUMLFormatSelectionWizardPage extends ToolkitWizardSelectionPage {

    private ExportUMLFormatSelectionControl selectExportFormat;
    ExportUML21WizardNode uml21WizardNode;
    ExportSaveFileWizardNode saveFileWizardNode;
    private static String titleLabel = "Please select the export format:";

    ExportUMLFormatSelectionWizardPage(ToolkitUIManager manager, String pageTitle) {
        super(manager, "ExportUMLFormatSelectionWizardPage", pageTitle, manager.getImageRegistry().getDescriptor(ToolkitUIManager.EXPORT));
        setDescription(ExportWizard.DESCRIPTION);
        uml21WizardNode = new ExportUML21WizardNode(manager);
        saveFileWizardNode = new ExportSaveFileWizardNode(manager);
    }

    public void createControl(Composite parent) {
        selectExportFormat = new ExportUMLFormatSelectionControl(parent, this, this.manager, titleLabel);
        setControl(selectExportFormat);

        selectExportFormat.setDefault();
        setSelectedNode(uml21WizardNode);
    }

    public void setParameters() {
        RhapsodyXMIToolkitParameters parameters = manager.getToolkit().getRhapsodyParameters();
        UMLVersionKind format = selectExportFormat.getSelectedMetamodelKind();
        if (format != null) {
            // Set UML Format
            parameters.setSelectedMetamodel(parameters.getMetamodelFromCommandLineFormat(format), format);
            String xmiWriter = selectExportFormat.getSelectedXMIWriter();
            if (xmiWriter != null) {
                // Set XMI Writer
                try {
                    parameters.setSelectedWriter(parameters.getSelectedMetamodel().getModelWriterDescriptor(xmiWriter));
                } catch (CoreException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        }
    }

    void setAdvancedProperties(RhapsodyXMIToolkitParameters parameters) {

        parameters.setExportMethods(selectExportFormat.getAdvancedPropertyValue(RhapsodyXMIToolkitParameters.KEY_EXPORT_METHODS));
        parameters.setExportStatemachines(selectExportFormat.getAdvancedPropertyValue(RhapsodyXMIToolkitParameters.KEY_EXPORT_STATEMACHINES));
        parameters.setExportCollaborations(selectExportFormat.getAdvancedPropertyValue(RhapsodyXMIToolkitParameters.KEY_EXPORT_COLLABORATIONS));
        parameters.setExportDeployments(selectExportFormat.getAdvancedPropertyValue(RhapsodyXMIToolkitParameters.KEY_EXPORT_DEPLOYMENTS));
        parameters.setExportProperties(selectExportFormat.getAdvancedPropertyValue(RhapsodyXMIToolkitParameters.KEY_EXPORT_PROPERTIES));
        parameters.setExportTags(selectExportFormat.getAdvancedPropertyValue(RhapsodyXMIToolkitParameters.KEY_EXPORT_TAGS));
        parameters.setExportAnnotations(selectExportFormat.getAdvancedPropertyValue(RhapsodyXMIToolkitParameters.KEY_EXPORT_ANNOTATIONS));
        parameters.setExportDescriptionHTML(selectExportFormat.getAdvancedPropertyValue(RhapsodyXMIToolkitParameters.KEY_EXPORT_DESCRIPTION_HTML));
        parameters.setExportDescriptionRTF(selectExportFormat.getAdvancedPropertyValue(RhapsodyXMIToolkitParameters.KEY_EXPORT_DESCRIPTION_RTF));
    }

    void setIsUML21Node(boolean is) {
        if (is) {
            setSelectedNode(uml21WizardNode);
        } else {
            setSelectedNode(saveFileWizardNode);
        }
    }

    public boolean isCommandLineFilled() {
        return this.manager.getToolkit().getParameters().isCommandLineProperty(RhapsodyXMIToolkitParameters.KEY_FORMAT);
    }
}
