package com.sodius.rhapsody.toolkit.ui;

import java.io.File;
import java.io.IOException;
import java.util.Calendar;
import java.util.GregorianCalendar;

import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.LocationEvent;
import org.eclipse.swt.browser.LocationListener;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.PaletteData;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

public class HelpBrowserHTML extends HelpSection{

	private int defaultWidth = 300;
	SelectionListener listener;

	private ToolkitUIManager manager;
	private Composite helpComposite;
	private File docFolder;
	private Image normal;
	private Image hover;
	private IContributionItem closeAction;

	HelpBrowserHTML(File file, ToolkitUIManager manager, Composite parent, int style) {
		super(parent, style);
		this.manager = manager;
		this.docFolder = file;
		this.setLayout(new GridLayout(1, false));
		this.setLayoutData(new GridData(GridData.FILL_VERTICAL));
		this.setBackground(this.getDisplay().getSystemColor(SWT.COLOR_WHITE));
		this.setBounds(0, 0, defaultWidth, this.getBounds().height);

		createContents();

		this.layout(true, true);
		this.pack();

	}

	private void createContents() {
		GridLayout layout = new GridLayout(1, true);
		layout.marginHeight=0;
		layout.marginTop = 0;
		layout.marginBottom = 0;
		layout.marginLeft=0;
		layout.marginRight=0;
		layout.marginWidth=0;
		
		Composite toolBarComposite = new Composite(this, SWT.NONE);
		toolBarComposite.setLayout(layout);
		toolBarComposite.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END ));

		ToolBar toolBar = new ToolBar(toolBarComposite, SWT.FLAT);
		toolBar.setBackground(this.getBackground());
		ToolBarManager toolBarManager = new ToolBarManager(toolBar);
		createActions();
		toolBarManager.add(closeAction);
		toolBarManager.update(true);
		helpComposite = new Composite(this, SWT.NONE);
		helpComposite.setLayout(new GridLayout(1, true));
		helpComposite.setLayoutData(new GridData(GridData.FILL_BOTH));
		helpComposite.setBackground(this.getDisplay().getSystemColor(SWT.COLOR_WHITE));

		Browser browser = new Browser(helpComposite, SWT.NONE);
		browser.setUrl(new File(docFolder, manager.getToolkit().getParameters().getCustomization().helpFileName).getAbsolutePath());
		browser.setLayoutData(new GridData(GridData.FILL_BOTH));
		browser.addLocationListener(new LocationListener() {
			
			public void changing(LocationEvent arg0) {
				if("pdf".equals(arg0.location.substring(arg0.location.lastIndexOf('.')+1)))
				{
					arg0.doit=false;
					try {
						BrowserControl.displayURL(arg0.location);
					} catch (IOException e) {
						e.printStackTrace();
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
			}
			
			public void changed(LocationEvent arg0) {
				// TODO Auto-generated method stub
			}
		});
		
		Label copyrightLabel = new Label(this, SWT.NONE);
		copyrightLabel.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		copyrightLabel.setText(" Sodius SAS, "+new GregorianCalendar().get(Calendar.YEAR));
		copyrightLabel.setBackground(this.getDisplay().getSystemColor(SWT.COLOR_WHITE));
		
		String version = manager.getToolkit().getParameters().getToolkitRelease();
		Label versionLabel = new Label(this, SWT.NONE);
		versionLabel.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		versionLabel.setText("version " + version);
		versionLabel.setBackground(this.getDisplay().getSystemColor(SWT.COLOR_WHITE));
	}

	public int getwidth() {
		return this.getBounds().width;
	}

	private void createImages() {
		Display display = Display.getCurrent();
		int[] shape = new int[] { 3, 3, 5, 3, 7, 5, 8, 5, 10, 3, 12, 3, 12, 5, 10, 7, 10, 8, 12, 10, 12, 12, 10, 12, 8, 10, 7, 10, 5, 12, 3, 12, 3, 10, 5, 8,
				5, 7, 3, 5 };

		/*
		 * Use magenta as transparency color since it is used infrequently.
		 */
		Color border = display.getSystemColor(SWT.COLOR_WIDGET_DARK_SHADOW);
		  Color background = display.getSystemColor(SWT.COLOR_LIST_BACKGROUND);
		  Color backgroundHot = new Color(display, new RGB(252, 160, 160));
		  Color transparent = display.getSystemColor(SWT.COLOR_MAGENTA);

		  PaletteData palette = new PaletteData(new RGB[] { transparent.getRGB(), border.getRGB(), background.getRGB(), backgroundHot.getRGB() });
		  ImageData data = new ImageData(16, 16, 8, palette);
		  data.transparentPixel = 0;

		  normal = new Image(display, data);
		  normal.setBackground(transparent);
		  GC gc = new GC(normal);
		  gc.setBackground(background);
		  gc.fillPolygon(shape);
		  gc.setForeground(border);
		  gc.drawPolygon(shape);
		  gc.dispose();

		  hover = new Image(display, data);
		  hover.setBackground(transparent);
		  gc = new GC(hover);
		  gc.setBackground(backgroundHot);
		  gc.fillPolygon(shape);
		  gc.setForeground(border);
		  gc.drawPolygon(shape);
		  gc.dispose();
		  
		backgroundHot.dispose();
	}

	private void createActions() {
		createImages();
		closeAction = new ContributionItem() {
			public void fill(ToolBar parent, int index) {
				final ToolItem item = new ToolItem(parent, SWT.PUSH);
				item.setImage(normal);
				item.setHotImage(hover);
				item.setToolTipText("close");
				item.addListener(SWT.Selection, new Listener() {
					public void handleEvent(org.eclipse.swt.widgets.Event event) {
						// close the tray
						manager.getDialog().closeTray();
						manager.getToolkit().getParameters().setHelpIsOpen(false);
					}
				});
			}
		};
	}

}

