package com.sodius.rhapsody.toolkit.ui;

import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import com.sodius.rhapsody.toolkit.properties.UMLVersionKind;

public class ImportLoadFileControl extends AdvancedBlocManager {

    private ModifyListener modifyListener;
    protected boolean wasInitialized = false;

    public ImportLoadFileControl(Composite parent, ImportLoadFileWizardPage page, ToolkitUIManager manager, String titleLabel) {
        super(parent, page, manager, titleLabel);
        this.setLayout(new GridLayout(1, false));
        this.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

        createContent();
    }

    @Override
    protected void createContent() {
        super.createContent();
        manageUMLVersion(); // @See RTC 124781
    }

    @Override
    protected void createFields() {
        rootComposite = new ShowFileControl(this, page, manager, titleLabel);
        title = ((ShowFileControl) rootComposite).getTitle();
    }

    @Override
    protected void createAndResetListener() {
        listener = new SelectionListener() {

            public void widgetDefaultSelected(SelectionEvent e) {
                // TODO Auto-generated method stub

            }

            public void widgetSelected(SelectionEvent e) {
                if ((advanced != null) && (e.getSource() == advanced)) {
                    if (advancedGroup == null) {
                        createAdvancedProperties();
                    } else if ((advancedGroup != null) && advancedGroup.isVisible()) {
                        clearAdvancedProperties();
                    } else {
                        showAdvancedProperties();
                    }
                    handleAdvancedButtonSelect();
                }
            }
        };

        modifyListener = new ModifyListener() {

            public void modifyText(ModifyEvent e) {
                manageUMLVersion();
            }
        };
    }

    protected void manageUMLVersion() {
        UMLVersionKind format = getMetamodelKind();
        if ((format != null) && UMLVersionKind.uml21.metamodelID.equals(format.metamodelID)) {
            page.setPageComplete(true);
            if (page instanceof ImportLoadFileWizardPage) {
                ((ImportLoadFileWizardPage) page).setIsUML21WizardNode(true);
            }
        } else {
            page.setPageComplete(false);
            if (page instanceof ImportLoadFileWizardPage) {
                ((ImportLoadFileWizardPage) page).setIsUML21WizardNode(false);
            }
        }

        if (advanced != null) {
            if ((advancedGroup != null) && advancedGroup.isVisible()) {
                clearAdvancedProperties();
            }
            clearAdvancedButton();
            handleAdvancedButtonSelect();
        }
    }

    public UMLVersionKind getMetamodelKind() {
        return ((ShowFileControl) rootComposite).getImportMetamodel();
    }

    @Override
    protected void registerListener() {
        ((ShowFileControl) rootComposite).getFileName().addModifyListener(modifyListener);

        // Register HelpListener on check buttons
        for (String propertyButtonName : properties.keySet()) {
            Button propertyButton = properties.get(propertyButtonName);
            propertyButton.addHelpListener(manager.getHelpListener());
        }
    }

    @Override
    protected void createAdvancedProperties() {
        // TODO Auto-generated method stub

    }
}
