package com.sodius.rhapsody.toolkit.util;

import com.sodius.mdw.log.EventStackLogger;
import com.sodius.mdw.log.EventStatus;
import com.sodius.mdw.log.context.ModelElementContext;
import com.sodius.mdw.log.reporter.tree.SubTreeGenerator;
import com.sodius.mdw.log.reporter.tree.Tree;

public class ReportTreeGenerator extends SubTreeGenerator {

	private final static String SORT_NAME="Events severity and Events kind";
	protected boolean hideUnknownCode;
	
	public ReportTreeGenerator() {
		super();
		this.hideUnknownCode=false;
	}
	
	@Override
	public boolean canGenerateSubTrees(Tree tree) {
		if((tree.getGroupLevel()<2))
		{
			if(hideUnknownCode)
			{
				for (EventStatus status : tree.getContainedStatus()) {
					if(!(status.getEventCode() instanceof EventStackLogger.UnknownCode))
						return true;
						
				}
			}
			else return true;
		}
		return false;
	}
	
	public ReportTreeGenerator(boolean displayUnnamedGroup, boolean hideUnknownCode) {
		super(displayUnnamedGroup);
		this.hideUnknownCode=hideUnknownCode;
	}

	@Override
	protected String getComment(String groupID, int groupLevel,
			EventStatus status) {
		switch (groupLevel) {
		case 0: // SEVERITY
			return status.getEventSeverity().getDescription();
		case 1: // EventKind
			return status.getEventCode().getKind().getDescription();
		case 2: // IsModelElement
			if(status.getContext() instanceof ModelElementContext)
				return "Event applied on an element of the model";
			else
				return "Event applied on an unknown step. Probably I/O.";
		}
		
		return "";
	}

	@Override
	protected String getGroupID(EventStatus status, int groupLevel) {
		if(hideUnknownCode && status.getEventCode() instanceof EventStackLogger.UnknownCode)
			return null;
		switch (groupLevel) {
		case 0: // SEVERITY
			return status.getEventSeverity().name();
		case 1: // EventKind
			return status.getEventCode().getKind().name();
		case 2: // IsModelElement
			if(status.getContext() instanceof ModelElementContext)
				return "Model";
			else
				return "Misc";
		}
		
		return "";
	}

	@Override
	public String getSortName() {
		return SORT_NAME;
	}
}
